(*
  This is ocaml-ofc, an OCaml interface for Open Flash Chart 2.

  Copyright (C) 2009  Florent Monnier
  Contact:  <fmonnier@linux-nantes.org>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published
  by the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*)


type chart    = (string * Json_type.json_type) list ref
type 'a elem  = (string * Json_type.json_type) list ref

type labels       = (string * Json_type.json_type)
type spoke_labels = (string * Json_type.json_type)

type label_item       = Json_type.json_type
type spoke_label_item = Json_type.json_type

type line
type area
type bar
type hbar
type bar_stack
type pie
type candle
type arrow


let new_chart() = ref []
let print_chart ?(print_header=true) j =
  if print_header then
    print_string "Content-type: application/json\r\n\r\n";
  print_endline(Json_io.string_of_json(Json_type.Object !j)) ;;

let chart_set_title j ?style title =
  let style = match style with None -> []
  | Some s -> ("style", Json_type.String("{" ^ s ^ "}")) :: [] in
  let title =
    ("title", Json_type.Object (("text", Json_type.String title)::style)) in
  j := title :: !j ;;

let chart_set_bg_colour j ~bg_colour =
  j := ("bg_colour", Json_type.String bg_colour) :: !j ;;


let chart_set_x_legend j ?style x_legend =
  let style = match style with None -> []
  | Some s -> ("style", Json_type.String("{" ^ s ^ "}")) :: [] in
  let x_legend =
    ("x_legend", Json_type.Object (("text", Json_type.String x_legend)::style)) in
  j := x_legend :: !j ;;

let chart_set_y_legend j ?style y_legend =
  let style = match style with None -> []
  | Some s -> ("style", Json_type.String("{" ^ s ^ "}")) :: [] in
  let y_legend =
    ("y_legend", Json_type.Object (("text", Json_type.String y_legend)::style)) in
  j := y_legend :: !j ;;



type align = Right | Center | Left
let string_of_align = function Right -> "right" | Center -> "center" | Left -> "left"

let _labels_attrs ?colour ?size ?steps ?visible_steps ?text ?align ?rotate ?visible () =
  let att = match colour with None -> []
  | Some col -> ("colour", Json_type.String col)::[] in
  let att = match size with None -> att
  | Some s -> ("size", Json_type.Int s)::att in
  let att = match steps with None -> att
  | Some s -> ("steps", Json_type.Int s)::att in
  let att = match visible_steps with None -> att
  | Some s -> ("visible-steps", Json_type.Int s)::att in
  let att = match text with None -> att
  | Some s -> ("text", Json_type.String s)::att in
  let att = match align with None -> att
  | Some s -> ("align", Json_type.String (string_of_align s))::att in
  let att = match rotate with None -> att
  | Some r -> ("rotate", Json_type.Int r)::att in
  let att = match visible with None -> att
  | Some b -> ("visible", Json_type.Bool b)::att in
  (att)

let _new_labels kind ?colour ?size ?steps ?visible_steps ?text ?align ?rotate ?visible labels =
  let att = _labels_attrs ?colour ?size ?steps ?visible_steps ?text ?align ?rotate ?visible () in
  let att =
    let l = List.map (fun s -> Json_type.String s) labels in
    ("labels", Json_type.Array l)::att in
  (kind, Json_type.Object att) ;;


let label_item ?colour ?size ?steps ?visible_steps ?align ?rotate ?visible text =
  let attrs = _labels_attrs ?colour ?size ?steps ?visible_steps ?text:None ?align ?rotate ?visible () in
  let attrs = ("text", Json_type.String text) :: attrs in
  (Json_type.Object attrs)


module Label = struct
  type lable_value = Int of int | Float of float
end

let x_label_item ?colour ?grid_colour ?size ?steps ?visible_steps ?text ?align ?rotate ?visible v =
  let attrs = _labels_attrs ?colour ?size ?steps ?visible_steps ?text ?align ?rotate ?visible () in
  let attrs = match grid_colour with None -> attrs
  | Some col -> ("grid-colour", Json_type.String col)::attrs in
  let attrs = (match v with Label.Int v -> ("x", Json_type.Int v)
                          | Label.Float v -> ("x", Json_type.Float v)) :: attrs in
  (Json_type.Object attrs)

let y_label_item ?colour ?grid_colour ?size ?steps ?visible_steps ?text ?align ?rotate ?visible v =
  let attrs = _labels_attrs ?colour ?size ?steps ?visible_steps ?text ?align ?rotate ?visible () in
  let attrs = match grid_colour with None -> attrs
  | Some col -> ("grid-colour", Json_type.String col)::attrs in
  let attrs = (match v with Label.Int v -> ("y", Json_type.Int v)
                          | Label.Float v -> ("y", Json_type.Float v)) :: attrs in
  (Json_type.Object attrs)



let _new_label_items kind items =
  let att = ("labels", Json_type.Array items)::[] in
  (kind, Json_type.Object att) ;;


let new_labels = _new_labels "labels" ;;
let new_spoke_labels = _new_labels "spoke-labels" ;;

let new_label_items = _new_label_items "labels" ;;
let new_spoke_label_items = _new_label_items "spoke-labels" ;;



let _chart_set_axis axis j ?min ?max ?steps ?colour ?grid_colour ?labels
                           ?stroke ?tick_size ?offset ?offset_b ?_3d () =
  let attrs = match min with None -> []
              | Some d -> ("min", Json_type.Int d)::[] in
  let attrs = match max with None -> attrs
              | Some d -> ("max", Json_type.Int d)::attrs in
  let attrs = match steps with None -> attrs
              | Some d -> ("steps", Json_type.Int d)::attrs in
  let attrs = match colour with None -> attrs
              | Some c -> ("colour", Json_type.String c)::attrs in
  let attrs = match grid_colour with None -> attrs
              | Some c -> ("grid-colour", Json_type.String c)::attrs in
  let attrs = match stroke with None -> attrs
              | Some d -> ("stroke", Json_type.Int d)::attrs in
  let attrs = match offset with None -> attrs
              | Some d -> ("offset", Json_type.Int d)::attrs in

  let attrs = match offset_b with None -> attrs
              | Some b -> ("offset", Json_type.Bool b)::attrs in

  let attrs = match _3d with None -> attrs
              | Some v -> ("3d", Json_type.Int v)::attrs in
  let attrs = match axis, tick_size with _, None -> attrs
              | "y_axis_right", Some s
              | "y_axis", Some s -> ("tick-length", Json_type.Int s)::attrs
              | "x_axis", Some s -> ("tick-height", Json_type.Int s)::attrs
              | _ -> assert false in
  let attrs = match labels with None -> attrs
  | Some labels -> labels :: attrs
  in
  let _axis = (axis, Json_type.Object attrs) in
  j := _axis :: !j ;;

let chart_set_x_axis = _chart_set_axis "x_axis" ;;
let chart_set_y_axis = _chart_set_axis "y_axis" ?_3d:None ;;
let chart_set_y_axis_right = _chart_set_axis "y_axis_right" ?_3d:None ;;


let chart_set_radar_axis j ?max ?steps ?colour ?grid_colour
                           ?labels ?spoke_labels ?stroke () =
  let att = match max with None -> []
  | Some v -> ("max", Json_type.Int v)::[] in
  let att = match steps with None -> att
  | Some v -> ("steps", Json_type.Int v)::att in
  let att = match colour with None -> att
  | Some col -> ("colour", Json_type.String col)::att in
  let att = match grid_colour with None -> att
  | Some col -> ("grid-colour", Json_type.String col)::att in
  let att = match stroke with None -> att
  | Some s -> ("stroke", Json_type.Int s)::att in
  let att = match labels with None -> att
  | Some labels -> labels :: att in
  let att = match spoke_labels with None -> att
  | Some spoke_labels -> spoke_labels :: att in
  j := ("radar_axis", Json_type.Object att) :: !j


(*
let chart_add_element j elem =
  let rec get_prev_elems = function
    | ("elements", Json_type.Array elems) :: _ -> elems
    | _ :: tl -> get_prev_elems tl
    | [] -> []
  in
  let prev_elems = get_prev_elems !j in
  let rec rem_prev_elems acc = function
    | ("elements", _) :: tl -> rem_prev_elems acc tl
    | hd :: tl -> rem_prev_elems (hd::acc) tl
    | [] -> (List.rev acc)
  in
  let _j = rem_prev_elems [] !j in
  let elem = (Json_type.Object(List.rev !elem)) in
  let el = ("elements", Json_type.Array (elem::prev_elems)) in
  j := el :: _j ;;
*)

let chart_add_element j elem =

  let elem = (Json_type.Object(List.rev !elem)) in

  let rec add_this_elem acc = function
    | false, ("elements", Json_type.Array elems) :: tl ->
        let el = ("elements", Json_type.Array (elems @ [elem])) in
        add_this_elem (el::acc) (true, tl)
    | true, ("elements", _) :: _ -> failwith "bug: several elements found"
    | was_replaced, hd::tl -> add_this_elem (hd::acc) (was_replaced, tl)
    | true, [] -> (List.rev acc)
    | false, [] ->
        let acc = ("elements", Json_type.Array [elem]) :: acc in
        (List.rev acc)
  in
  j := add_this_elem [] (false, !j) ;;


type v_elem = Area of area elem | Line of line elem | Pie of pie elem
            | Bar of bar elem | HBar of hbar elem | BarStack of bar_stack elem
            | Arrow of arrow elem | Candle of candle elem

let chart_add_elements j elems =
  let cast_elem (e : 'a elem) = (Obj.magic e : 'b elem) in
  let elems = List.map (function Line e -> cast_elem e
                               | Area e -> cast_elem e
                               | Pie  e -> cast_elem e
                               | Bar  e -> cast_elem e
                               | HBar e -> cast_elem e
                               | BarStack e -> cast_elem e
                               | Arrow  e -> cast_elem e
                               | Candle e -> cast_elem e) elems in
  let elems = List.map (fun elem -> Json_type.Object(List.rev !elem)) elems in

  let rec add_these_elems acc = function
    | false, ("elements", Json_type.Array prev_elems) :: tl ->
        let el = ("elements", Json_type.Array (elems @ prev_elems)) in
        add_these_elems (el::acc) (true, tl)
    | true, ("elements", _) :: _ -> failwith "bug: several elements found"
    | was_replaced, hd::tl -> add_these_elems (hd::acc) (was_replaced, tl)
    | true, [] -> (List.rev acc)
    | false, [] ->
        let acc = ("elements", Json_type.Array elems) :: acc in
        (List.rev acc)
  in
  j := add_these_elems [] (false, !j) ;;


let chart_set_tooltip j ?colour ?background ?title ?body ?stroke ?shadow ?mouse () =
  let att = match colour with None -> []
  | Some col -> ("colour", Json_type.String col)::[] in
  let att = match background with None -> att
  | Some col -> ("background", Json_type.String col)::att in
  let att = match title with None -> att
  | Some css -> ("title", Json_type.String ("{" ^ css ^ "}"))::att in
  let att = match body with None -> att
  | Some css -> ("body", Json_type.String ("{" ^ css ^ "}"))::att in
  let att = match mouse with None -> att
  | Some d -> ("mouse", Json_type.Int d)::att in
  let att = match stroke with None -> att
  | Some s -> ("stroke", Json_type.Int s)::att in
  let att = match shadow with None -> att
  | Some s -> ("shadow", Json_type.Bool s)::att in
  j := ("tooltip", Json_type.Object att) :: !j ;;



(* Line Chart *)

let new_line_elem() =
  ref [("type", Json_type.String "line")] ;;

let line_elem_set_width lc v =
  lc := ("width", Json_type.Int v) :: !lc ;;

let line_elem_set_colour lc col =
  lc := ("colour", Json_type.String col) :: !lc ;;

let line_elem_set_dot_size lc size =
  lc := ("dot-size", Json_type.Int size) :: !lc ;;

let line_elem_set_halo_size lc size =
  lc := ("halo-size", Json_type.Int size) :: !lc ;;

let line_elem_set_legend lc str =
  lc := ("text", Json_type.String str) :: !lc ;;

let line_elem_set_font_size lc size =
  lc := ("font-size", Json_type.Int size) :: !lc ;;

let line_elem_set_loop lc loop =
  lc := ("loop", Json_type.Bool loop) :: !lc ;;

let line_elem_add_ints lc va =
  let ja = List.map (fun v -> Json_type.Int v) va in
  lc := ("values", Json_type.Array ja) :: !lc ;;

let line_elem_add_floats lc va =
  let ja = List.map (fun v -> Json_type.Float v) va in
  lc := ("values", Json_type.Array ja) :: !lc ;;

type dot_value = I of int | F of float | II of int * int | FF of float * float
               | IF of int * float | FI of float * int
type dot = (string * Json_type.json_type) list
type value = Int of int | Float of float | Value of dot | Null

type dot_kind = Dot | Solid_dot | Hollow_dot | Anchor | Star | Bow
let dot_kind = function
  | Dot        -> "dot"
  | Solid_dot  -> "solid-dot"
  | Hollow_dot -> "hollow-dot"
  | Anchor     -> "anchor"
  | Star       -> "star"
  | Bow        -> "bow"

let dot ?colour ?kind ?tip ?dot_size ?halo_size ?sides
        ?label ?label_colour ?font_size ?on_click
        dot_value =
  let att = match colour with None -> []
  | Some c -> ("colour", Json_type.String c)::[] in
  let att = match kind with None -> att
  | Some k -> ("type", Json_type.String (dot_kind k))::att in
  let att = match tip with None -> att
  | Some str -> ("tip", Json_type.String str)::att in
  let att = match dot_size with None -> att
  | Some s -> ("dot-size", Json_type.Int s)::att in
  let att = match halo_size with None -> att
  | Some s -> ("halo-size", Json_type.Int s)::att in
  let att = match sides with None -> att
  | Some s -> ("sides", Json_type.Int s)::att in
  let att = match on_click with None -> att
  | Some str -> ("on-click", Json_type.String str)::att in
  let att = match label with None -> att
  | Some str -> ("label", Json_type.String str)::att in
  let att = match font_size with None -> att
  | Some s -> ("font-size", Json_type.Int s)::att in
  let att = match label_colour with None -> att
  | Some c -> ("label-colour", Json_type.String c)::att in
  let att = match dot_value with
  | I d -> ("value", Json_type.Int d)::att
  | F g -> ("value", Json_type.Float g)::att
  | II(x, y) -> ("x", Json_type.Int   x)::("y", Json_type.Int   y)::att
  | FF(x, y) -> ("x", Json_type.Float x)::("y", Json_type.Float y)::att
  | IF(x, y) -> ("x", Json_type.Int   x)::("y", Json_type.Float y)::att
  | FI(x, y) -> ("x", Json_type.Float x)::("y", Json_type.Int   y)::att in
  (Value att)


let line_elem_add_values lc va =
  let f = (function Null -> Json_type.Null
                  | Int v -> Json_type.Int v
                  | Float v -> Json_type.Float v
                  | Value dot -> Json_type.Object dot) in
  let ja = List.map f va in
  lc := ("values", Json_type.Array ja) :: !lc ;;


let line_elem_set_dot_style lc ?kind ?hollow ?colour ?alpha
                               ?background_colour ?background_alpha
                               ?dot_size ?size ?sides ?width ?tip ?halo_size
                               () =
  let att = match kind with None -> []
  | Some k -> ("type", Json_type.String(dot_kind k))::[] in
  let att = match hollow with None -> att
  | Some b -> ("hollow", Json_type.Bool b)::att in
  let att = match colour with None -> att
  | Some c -> ("colour", Json_type.String c)::att in
  let att = match alpha with None -> att
  | Some f -> ("alpha", Json_type.Float f)::att in
  let att = match background_colour with None -> att
  | Some c -> ("background-colour", Json_type.String c)::att in
  let att = match background_alpha with None -> att
  | Some f -> ("background-alpha", Json_type.Float f)::att in
  let att = match dot_size with None -> att
  | Some s -> ("dot-size", Json_type.Int s)::att in
  let att = match sides with None -> att
  | Some s -> ("sides", Json_type.Int s)::att in
  let att = match size with None -> att
  | Some s -> ("size", Json_type.Int s)::att in
  let att = match width with None -> att
  | Some s -> ("width", Json_type.Int s)::att in
  let att = match tip with None -> att
  | Some str -> ("tip", Json_type.String str)::att in
  let att = match halo_size with None -> att
  | Some s -> ("halo-size", Json_type.Int s)::att in
  let dot_style =
    ("dot-style", Json_type.Object att) in
  lc := dot_style :: !lc ;;


type line_style = Dash | Solid

let line_style = function
  | Dash  -> "dash"
  | Solid -> "solid"

let line_elem_set_line_style lc ?style ?on ?off () =
  let att = match style with None -> []
  | Some s -> ("style", Json_type.String (line_style s))::[] in
  let att = match on with None -> att
  | Some d -> ("on", Json_type.Int d)::att in
  let att = match off with None -> att
  | Some d -> ("off", Json_type.Int d)::att in
  let line_style =
    ("line-style", Json_type.Object att) in
  lc := line_style :: !lc ;;

type on_show_kind =
  | Mid_slide
  | Grow_up
  | Pop
  | Pop_up
  | Drop
  | Shrink_in
  | Explode
  | Fade_in

let on_show_kind = function
  | Mid_slide -> "mid-slide"
  | Grow_up   -> "grow-up"
  | Pop       -> "pop"
  | Pop_up    -> "pop-up"
  | Drop      -> "drop"
  | Shrink_in -> "shrink-in"
  | Explode   -> "explode"
  | Fade_in   -> "fade-in"

let line_elem_set_on_show lc ~kind ?cascade ?delay () =
  let att = match cascade with None -> []
  | Some v -> ("cascade", Json_type.Float v)::[] in
  let att = match delay with None -> []
  | Some v -> ("delay", Json_type.Float v)::att in

  let att = ("type", Json_type.String (on_show_kind kind))::att in
  let on_show =
    ("on-show", Json_type.Object att) in
  lc := on_show :: !lc ;;



(* Area Chart *)

let new_area_elem() =
  ref [("type", Json_type.String "area")] ;;

let area_elem_set_fill ac fill =
  ac := ("fill", Json_type.String fill) :: !ac ;;

let area_elem_set_fill_alpha ac fill_alpha =
  ac := ("fill-alpha", Json_type.Float fill_alpha) :: !ac ;;

(**)
let area_of_line (e : line elem) = (Obj.magic e : area elem) ;;
let area_elem_set_width      e = line_elem_set_width      (area_of_line e) ;;
let area_elem_set_colour     e = line_elem_set_colour     (area_of_line e) ;;  
let area_elem_set_dot_size   e = line_elem_set_dot_size   (area_of_line e) ;;
let area_elem_set_halo_size  e = line_elem_set_halo_size  (area_of_line e) ;;
let area_elem_set_legend     e = line_elem_set_legend     (area_of_line e) ;;
let area_elem_set_font_size  e = line_elem_set_font_size  (area_of_line e) ;;
let area_elem_set_loop       e = line_elem_set_loop       (area_of_line e) ;;
let area_elem_set_dot_style  e = line_elem_set_dot_style  (area_of_line e) ;;
let area_elem_set_line_style e = line_elem_set_line_style (area_of_line e) ;;
let area_elem_set_on_show    e = line_elem_set_on_show    (area_of_line e) ;;
let area_elem_add_ints       e = line_elem_add_ints       (area_of_line e) ;;
let area_elem_add_floats     e = line_elem_add_floats     (area_of_line e) ;;
let area_elem_add_values     e = line_elem_add_values     (area_of_line e) ;;
(**)



(* Bar element *)

let new_bar_elem() = ref [("type", Json_type.String "bar")] ;;

let new_bar_sketch_elem()           = ref [("type", Json_type.String "bar_sketch")] ;;
let new_bar_3d_elem()               = ref [("type", Json_type.String "bar_3d")] ;;
let new_bar_filled_elem()           = ref [("type", Json_type.String "bar_filled")] ;;
let new_bar_cylinder_elem()         = ref [("type", Json_type.String "bar_cylinder")] ;;
let new_bar_cylinder_outline_elem() = ref [("type", Json_type.String "bar_cylinder_outline")] ;;
let new_bar_round_glass_elem()      = ref [("type", Json_type.String "bar_round_glass")] ;;
let new_bar_round_elem()            = ref [("type", Json_type.String "bar_round")] ;;
let new_bar_dome_elem()             = ref [("type", Json_type.String "bar_dome")] ;;
let new_bar_glass_elem()            = ref [("type", Json_type.String "bar_glass")] ;;
let new_bar_plastic_elem()          = ref [("type", Json_type.String "bar_plastic")] ;;
let new_bar_plastic_flat_elem()     = ref [("type", Json_type.String "bar_plastic_flat")] ;;
let new_bar_round3d_elem()          = ref [("type", Json_type.String "bar_round3d")] ;;


let bar_elem_set_alpha bc ~alpha =
  bc := ("alpha", Json_type.Float alpha) :: !bc ;;

let bar_elem_set_tip bc ~tip =
  bc := ("tip", Json_type.String tip) :: !bc ;;

(**)
let bar_of_line (e : line elem) = (Obj.magic e : bar elem) ;;
let bar_elem_set_width      e = line_elem_set_width      (area_of_line e) ;;
let bar_elem_set_colour     e = line_elem_set_colour     (area_of_line e) ;;  
let bar_elem_set_dot_size   e = line_elem_set_dot_size   (area_of_line e) ;;
let bar_elem_set_legend     e = line_elem_set_legend     (area_of_line e) ;;
let bar_elem_set_font_size  e = line_elem_set_font_size  (area_of_line e) ;;
let bar_elem_set_on_show    e = line_elem_set_on_show    (area_of_line e) ;;
let bar_elem_add_ints       e = line_elem_add_ints       (area_of_line e) ;;
let bar_elem_add_floats     e = line_elem_add_floats     (area_of_line e) ;;
let bar_elem_add_values     e = line_elem_add_values     (area_of_line e) ;;
(**)


(* Horizontal-Bar element *)

let new_hbar_elem() = ref [("type", Json_type.String "hbar")] ;;

let hbar_elem_set_tip hb ~tip =
  hb := ("tip", Json_type.String tip) :: !hb ;;

(**)
let hbar_of_line (e : line elem) = (Obj.magic e : hbar elem) ;;
let hbar_elem_set_width      e = line_elem_set_width      (hbar_of_line e) ;;
let hbar_elem_set_colour     e = line_elem_set_colour     (hbar_of_line e) ;;  
let hbar_elem_set_dot_size   e = line_elem_set_dot_size   (hbar_of_line e) ;;
let hbar_elem_set_legend     e = line_elem_set_legend     (hbar_of_line e) ;;
let hbar_elem_set_font_size  e = line_elem_set_font_size  (hbar_of_line e) ;;
let hbar_elem_set_on_show    e = line_elem_set_on_show    (hbar_of_line e) ;;
(**)


let hbar_elem_add_ints lc va =
  let ja = List.map (fun v -> Json_type.Object [("right", Json_type.Int v)]) va in
  lc := ("values", Json_type.Array ja) :: !lc ;;

let hbar_elem_add_floats lc va =
  let ja = List.map (fun v -> Json_type.Object [("right", Json_type.Float v)]) va in
  lc := ("values", Json_type.Array ja) :: !lc ;;


module HBar_values = struct
  (* TODO: other params *)
  type hbar_value = Int of int | Float of float
                  | LeftRightInt of int * int | LeftRightFloat of float * float
end

let hbar_elem_add_values lc va =
  let f = (function
  | HBar_values.Int v -> Json_type.Object [("right", Json_type.Int v)]
  | HBar_values.Float v -> Json_type.Object [("right", Json_type.Float v)]
  | HBar_values.LeftRightInt(l, r) -> Json_type.Object [("left", Json_type.Int l);
                                                        ("right", Json_type.Int r)]
  | HBar_values.LeftRightFloat(l, r) -> Json_type.Object [("left", Json_type.Float l);
                                                          ("right", Json_type.Float r)]
  ) in
  let ja = List.map f va in
  lc := ("values", Json_type.Array ja) :: !lc ;;



(* Bar-Stack element *)

let new_bar_stack_elem() = ref [("type", Json_type.String "bar_stack")] ;;


let bar_stack_elem_set_colours bs ~colours =
  let colours = List.map (fun c -> Json_type.String c) colours in
  bs := ("colours", Json_type.Array colours) :: !bs ;;


let bar_stack_elem_add_ints bs vll =
  let f = (function v -> Json_type.Int v) in
  let fll = (function
    | [] -> Json_type.Null
    | vl -> Json_type.Array (List.map f vl)
  ) in
  let jll = List.map fll vll in
  bs := ("values", Json_type.Array jll) :: !bs ;;


let bar_stack_elem_add_floats bs vll =
  let f = (function v -> Json_type.Float v) in
  let fll = (function
    | [] -> Json_type.Null
    | vl -> Json_type.Array (List.map f vl)
  ) in
  let jll = List.map fll vll in
  bs := ("values", Json_type.Array jll) :: !bs ;;


module Bar_stack_values = struct
  type attr_value = (string * Json_type.json_type) list
  type value = Int of int | Float of float | Value of attr_value | Null
end

let bs_value_int ?colour v =
  let att = match colour with None -> []
  | Some col -> ("colour", Json_type.String col)::[] in
  Bar_stack_values.Value(("val", Json_type.Int v)::att) ;;

let bs_value_float ?colour v =
  let att = match colour with None -> []
  | Some col -> ("colour", Json_type.String col)::[] in
  Bar_stack_values.Value(("val", Json_type.Float v)::att) ;;

let bar_stack_elem_add_values bs vll =
  let f = (function Bar_stack_values.Int v -> Json_type.Int v
                  | Bar_stack_values.Float v -> Json_type.Float v
                  | Bar_stack_values.Null -> Json_type.Null
                  | Bar_stack_values.Value v -> Json_type.Object v) in
  let fll = (function
    | [] -> Json_type.Null
    | vl -> Json_type.Array (List.map f vl)
  ) in
  let jll = List.map fll vll in
  bs := ("values", Json_type.Array jll) :: !bs ;;


type bar_stack_key = Json_type.json_type

let bs_key ?colour ?text ?font_size () =
  let att = match colour with None -> []
  | Some col -> ("colour", Json_type.String col)::[] in
  let att = match text with None -> att
  | Some s -> ("text", Json_type.String s)::att in
  let att = match font_size with None -> att
  | Some s -> ("font-size", Json_type.Int s)::att in
  (Json_type.Object att) ;;


let bar_stack_elem_set_keys bs keys =
  bs := ("keys", Json_type.Array keys) :: !bs ;;



(* Pie Chart *)

let new_pie_elem() =
  ref [("type", Json_type.String "pie")] ;;

let pie_elem_set_colours pc ~colours =
  let colours = List.map (fun c -> Json_type.String c) colours in
  pc := ("colours", Json_type.Array colours) :: !pc ;;

let pie_elem_set_radius pc ~radius =
  pc := ("radius", Json_type.Int radius) :: !pc ;;

let pie_elem_set_alpha pc ~alpha =
  pc := ("alpha", Json_type.Float alpha) :: !pc ;;

let pie_elem_set_border pc ~border =
  pc := ("border", Json_type.Int border) :: !pc ;;

let pie_elem_set_start_angle pc ~start_angle =
  pc := ("start-angle", Json_type.Int start_angle) :: !pc ;;

let pie_elem_set_gradient_fill pc ~gradient_fill =
  pc := ("gradient-fill", Json_type.Bool gradient_fill) :: !pc ;;

let pie_elem_set_no_labels pc ~no_labels =
  pc := ("no-labels", Json_type.Bool no_labels) :: !pc ;;

let pie_elem_set_label_colour pc ~label_colour =
  pc := ("label-colour", Json_type.String label_colour) :: !pc ;;

let pie_elem_set_tip pc ~tip =
  pc := ("tip", Json_type.String tip) :: !pc ;;

let pie_elem_set_animate pc ~animate =
  pc := ("animate", Json_type.Bool animate) :: !pc ;;

type pie_animations = Bounce of int | Fade

let pie_elem_set_animations pc ~animations =
  let f = function
    | Bounce distance ->
        Json_type.Object
          [("type", Json_type.String "bounce");
           ("distance", Json_type.Int 8)]
    | Fade ->
        Json_type.Object [("type", Json_type.String "fade")]
  in
  let attrs = List.map f animations in
  pc := ("animate", Json_type.Array attrs) :: !pc ;;


module Pie_values = struct
  type pie_value_attrs = (string * Json_type.json_type) list
  type pie_value = Int of int | Float of float | Value of pie_value_attrs | Null
  type pie_value_param = I of int | F of float | Zero
end

let pie_value ?label ?tip ?on_click v =
  let att = match label with None -> []
  | Some l -> ("label", Json_type.String l)::[] in
  let att = match tip with None -> att
  | Some t -> ("tip", Json_type.String t)::att in
  let att = match on_click with None -> att
  | Some s -> ("on-click", Json_type.String s)::att in
  let att = match v with
  | Pie_values.I d -> ("value", Json_type.Int d)::att
  | Pie_values.F f -> ("value", Json_type.Float f)::att
  | Pie_values.Zero -> ("value", Json_type.Null)::att
  in
  (Pie_values.Value att)


let pie_elem_add_values pc va =
  let f = (function Pie_values.Null -> Json_type.Null
                  | Pie_values.Int v -> Json_type.Int v
                  | Pie_values.Float v -> Json_type.Float v
                  | Pie_values.Value v -> Json_type.Object v) in
  let ja = List.map f va in
  pc := ("values", Json_type.Array ja) :: !pc ;;


(* Candle Element *)

let new_candle_elem() =
  ref [("type", Json_type.String "candle")] ;;

let candle_of_line (e : line elem) = (Obj.magic e : candle elem) ;;
let candle_elem_set_on_show e = line_elem_set_on_show (candle_of_line e) ;;


let candle_elem_set_colour cc ~colour =
  cc := ("colour", Json_type.String colour) :: !cc ;;

let candle_elem_set_negative_colour cc ~colour =
  cc := ("negative-colour", Json_type.String colour) :: !cc ;;

let candle_elem_set_alpha cc ~alpha =
  cc := ("alpha", Json_type.Float alpha) :: !cc ;;

let candle_elem_set_legend cc ~text =
  cc := ("text", Json_type.String text) :: !cc ;;

let candle_elem_set_font_size cc ~size =
  cc := ("font-size", Json_type.Int size) :: !cc ;;

let candle_elem_set_tip cc ~tip =
  cc := ("tip", Json_type.String tip) :: !cc ;;


type candle_value = Json_type.json_type

let candle_value_ints ~high ~top ~bottom ~low ?tip ?colour () =
  let att =
    [ ("high", Json_type.Int high); ("top", Json_type.Int top);
      ("bottom", Json_type.Int bottom); ("low", Json_type.Int low) ] in
  let att = match tip with None -> att
  | Some tip -> ("tip", Json_type.String tip)::att in
  let att = match colour with None -> att
  | Some col -> ("colour", Json_type.String col)::att in
  (Json_type.Object att) ;;

let candle_value_floats ~high ~top ~bottom ~low ?tip ?colour () =
  let att =
    [ ("high", Json_type.Float high); ("top", Json_type.Float top);
      ("bottom", Json_type.Float bottom); ("low", Json_type.Float low) ] in
  let att = match tip with None -> att
  | Some tip -> ("tip", Json_type.String tip)::att in
  let att = match colour with None -> att
  | Some col -> ("colour", Json_type.String col)::att in
  (Json_type.Object att) ;;


let candle_elem_add_values cc va =
  cc := ("values", Json_type.Array va) :: !cc ;;



(* Arrow Element *)

let new_arrow_elem ~start ~end_ ?barb_length ?colour () =
  let att = ("type", Json_type.String "arrow")::[] in
  let att = match start with
  | x, y -> ("start", Json_type.Object [("x", Json_type.Int x);
                                        ("y", Json_type.Int y)])::att in
  let att = match end_ with
  | x, y -> ("end", Json_type.Object [("x", Json_type.Int 4);
                                      ("y", Json_type.Int 7)])::att in

  let att = match barb_length with None -> att
  | Some v -> ("barb-length", Json_type.Int v)::att in
  let att = match colour with None -> att
  | Some col -> ("colour", Json_type.String col)::att in
  (ref att) ;;


(* vim: ts=2 sts=2 sw=2 et fdm=marker
 *)
